import time
import os
import sys
import bz2
import xml.sax
import mwparserfromhell

ROOT_DIR = os.path.dirname(os.path.abspath("wikiextractor.py"))
dbinsertion_path=ROOT_DIR+"\paniniwikiparser\wiki\db"
common_path=ROOT_DIR+"\paniniwikiparser\common"
data_path=ROOT_DIR+"\paniniwikiparser\wiki\data"

sys.path.insert(0,dbinsertion_path)
from dbinsertion import *

sys.path.insert(0,common_path)
from preprocesstext import preprocess_text

output=[]
class stack:
    def __init__ (self):
        self.items = []

    def push(self, item):
        self.items.append(item)

    def pop(self):
        if self.items:
            return self.items.pop()
        else:
            raise Exception("stack is empty")

    def top(self):
        return self.items[-1] if self.items else None

class WikiXmlHandler(xml.sax.handler.ContentHandler):
    """Parse through XML data using SAX"""
    def __init__(self, ns = [0]):
        xml.sax.handler.ContentHandler.__init__(self)
        self._buffer = None
        self._values = {}
        self._current_tag = None
        self._pages = []
        self._ns = ns
        self._stack =  stack()

    def characters(self, content):
        """Characters between opening and closing tags"""
        if self._current_tag:
            self._buffer.append(content)


    def startElement(self, name, attrs):
        """Opening tag of element"""
        self._stack.push(name)
        
        if name =="page":
            self._values["redirect"]=""
            self._values["parentid"]=""
            self._values["revisionid"] =""

        if name in ('title', 'text', 'timestamp', 'ns', 'id',"parentid","redirect"):
            self._current_tag = name
            self._buffer = []

            if name == 'redirect':
                self._values[name] = attrs["title"] #for rediret

    def endElement(self, name):
        """Closing tag of element"""
        self._stack.pop()
        if name == self._current_tag:
            if name == "id":
                if self._stack.top() =="page":
                    v = ' '.join(self._buffer)
                    if isinstance(v, str):
                        self._values[name] = int(v)  # For PageID 
                if self._stack.top() =="revision":
                    v = ' '.join(self._buffer)
                    if isinstance(v, str):
                        self._values["revisionid"] = int(v)  #For revision ID 
            elif name != "redirect":
                self._values[name] = ' '.join(self._buffer)
        if name == 'page' and self._values['ns'] and int(self._values['ns']) in self._ns:
            self._pages.append(
                                (self._values['title'],self._values['text'],self._values['id'],
                                int(self._values['ns']),self._values["redirect"],self._values["parentid"],
                                self._values["revisionid"]
                                )
                            )
    
def proces_xml_page(page):
    ''' this will accept parsed wiki data and process it and insert it into db'''
    global output
    #print(f'Title is {page[0]}')  #0- title, 1- text,2- id, 3- ns,
                                    #4- redirect,5-parentid,6-revisionid
    title=page[0]
    raw_text=page[1]
    page_id=page[2]
    ns=page[3]
    redirect=page[4]
    parentid=page[5]
    revisionid=page[6]

    #Collect page text
    wiki = mwparserfromhell.parse(raw_text)

    #clean text
    clean_text=wiki.strip_code().strip()

    #pre processing using nltk
    preprocessed_text=preprocess_text(clean_text)

    #collect Internal links in wiki page
    wikilinks = [(x.title, x.text) for x in wiki.filter_wikilinks()]

    #collect External links in wiki page
    external_links = [(x.title, x.url) for x in wiki.filter_external_links()]
    
    isredirect=0
    if(len(redirect)!=0):
        isredirect=1

    if title!="":
        if len(title)<=40 and len(preprocessed_text)>100 and ns==0 and len(wikilinks)>5 and len(external_links)>5:
            #Intiating db insertion 
            load_articles(
            {
                    "page_id":page_id,
                    "Page_title":title,
                    "ns":ns,
                    "raw_text":raw_text,
                    "clean_text":clean_text,
                    "processed_text":preprocessed_text,
                    "redirect":redirect,
                    "parentid":parentid,
                    "revision_id":revisionid,
                    "isredirect":isredirect,
                    "isdeleted":0
            })
        else:
            #Intiating db insertion 
            load_articles(
            {
                    "page_id":page_id,
                    "Page_title":title,
                    "ns":ns,
                    "raw_text":raw_text,
                    "clean_text":clean_text,
                    "processed_text":preprocessed_text,
                    "redirect":redirect,
                    "parentid":parentid,
                    "revision_id":revisionid,
                    "isredirect":isredirect,
                    "isdeleted":1
            })
    

    # if preprocessed_text!="":
    #     obj={
    #     "page_id":page[2],
    #     "Page_title":page[0],
    #     "ns":page[3],
    #     "raw_text":page[1],
    #     "clean_text":clean_text,
    #     "processed_text":preprocessed_text,
    #     "redirect":page[4],
    #     "parentid":page[5],
    #     "revision_id":page[6],
    #     "isdeleted":page[3]
    #     }
    #     output.append(obj)
    
def readxml_data(file_name):

    try:
        index=0
        start_time = time.time()

        handler = WikiXmlHandler()
        
        # Parsing object
        parser = xml.sax.make_parser()
        parser.setContentHandler(handler)

        for i, line in enumerate(bz2.BZ2File(file_name, 'r')):
            parser.feed(line)
            if len(handler._pages)>0:
                proces_xml_page(handler._pages[0])
                index=index+1
                handler._pages=[]
            # if(index>5):
            #     res=json.dumps(output,default=vars)
            #     with open('result_clean_markup.json','w') as f:
            #         json.dump(res,f)
            #         break;

        elapsed_time = time.time() - start_time
        print("file reading finished")
        print(elapsed_time);
    except Exception as e:
        print(e)



import nltk

from nltk.corpus import stopwords
from nltk.tokenize import word_tokenize
from dateutil import parser

# Need to preprocess to check for stopwords. Need to load here as we cant load from setup.py
nltk.download('stopwords')
nltk.download("punkt")

def is_valid_date(date_str):
    try:
        parser.parse(date_str)
        return True
    except:
        return False

#preprocessing
def preprocess_text(input_text):
    processed_text=""
    try:
        #Collecting  stop_words from NLTK library
        stop_words = set(stopwords.words('english'))
        word_tokens = word_tokenize(input_text)

        #Remove stop words in content
        non_stopwords = [w.lower() for w in word_tokens if not w in stop_words]

        if len(non_stopwords)>100:

            #Remove date related information in 'text'
            words=[' '.join([w for w in line.split() if not is_valid_date(w)]) for line in non_stopwords]
            text=' '.join(words)
            #print(text)
            
            #Remove number, contains number, separate punctuation
            tokenizer = nltk.RegexpTokenizer(r"\b[a-zA-Z][a-zA-Z]+\b")
            new_words=tokenizer.tokenize(text)
            processed_text=' '.join(new_words)
            #print(processed_text)
        return processed_text
    except Exception as e:
        return processed_text
        print(e.args)


import pymysql 
from db_config import HOST,USER,PASSWORD,DATABASE_NAME

def isfile_already_read(filename):
    try:
        conn  = pymysql.connect(host=HOST, user=USER, password=PASSWORD,db=DATABASE_NAME) 
        # Create a cursor object 
        cur  = conn.cursor() 
        try:
            sql="select filename from fileread_status where filename=%s"
            param=(filename)
            cur.execute(sql,param)
            myresult = cur.fetchall()
            if(len(myresult)>0):
                return True
            else:
                return False
        except Exception as e :
            print(e)
            pass
        print(f"{cur.rowcount} details inserted before preprocess ") 
        conn.commit() 
        conn.close()
    except Exception as e:
        print(e.args)
        return False

def insert_filestatus(filename):
    try:
        conn  = pymysql.connect(host=HOST, user=USER, password=PASSWORD,db=DATABASE_NAME) 
        # Create a cursor object 
        cur  = conn.cursor() 
        query = f"INSERT INTO fileread_status (filename) VALUES ('{filename}')"
        try:
            cur.execute(query) 
        except Exception as e:
            print(e.args)
        conn.commit() 
        conn.close()
    except Exception as e:
        print(e.args)

def load_articles(article):
    try:
        conn  = pymysql.connect(host=HOST, user=USER, password=PASSWORD,db=DATABASE_NAME) 
        # Create a cursor object 
        cur  = conn.cursor() 
        try:
            page_title = article["Page_title"].replace("'","")
            page_id = article["page_id"]
            text = article["processed_text"]
            isredirect= article["isredirect"]
            redirect_title=article["redirect"]
            revision_id=article["revision_id"]
            isdeleted=article["isdeleted"]
        except Exception as e :
            print(e)
            pass
        query = f"INSERT INTO article (page_title, page_id, content, isredirect, redirect_title, revisionid,isdeleted  ) VALUES ('{page_title}', '{page_id}', '{text}','{isredirect}','{redirect_title}','{revision_id}','{isdeleted}')"
        try:
            cur.execute(query) 
        except Exception as e:
            print(e.args)
            pass
        #print(f"{cur.rowcount} details inserted after cleaning") 
        conn.commit() 
        conn.close()
    except Exception as e:
        print(e.args)



# IP address of the MySQL database server 
HOST="panini-wiki.mariadb.database.azure.com"
# User name of the database server 
USER = "wikiadmin@panini-wiki"
# Password for the database user 
PASSWORD = "Pass@word2"
DATABASE_NAME = "wiki_temp"
ACCOUNT_NAME = "allmixedfilesforsearch"
ACCOUNT_KEY= "byw2umVaRocXLoDCDqkQBEmMl4IAbC1wBrRMGQq6b0H5DhIuneE/BJoz/FTVYIaGN1eI9K8NPjO00sFpEjt7NQ=="
CONTAINER_NAME="wiki"



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="paniniwikiparser", # Replace with your own username
    version="0.0.1",
    author="Anil Vemula",
    author_email="v-anvemu@microsoft.com",
    description="Parses wiki xml",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://o365exchange.visualstudio.com/O365%20Sandbox/_git/IPMLExp?path=%2Fsrc%2FPanini2%2FSyntheticDataGenerator&version=GBmaster",
    packages=setuptools.find_packages(),
    install_requires=['six','nltk','python-dateutil','nameparser','PyMySQL','mwparserfromhell '],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)